==> conf-ini-g-2023.25.tar.gz: conf-ini-g-2023.25/conf_ini_g/phase/typed/section/units.py <==
# Copyright CNRS/Inria/UNS
# Contributor(s): Eric Debreuve (since 2021)
#
# eric.debreuve@cnrs.fr
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from conf_ini_g.interface.storage.parameter import INI_COMMENT_MARKER
from conf_ini_g.interface.storage.section import INI_UNIT_SECTION
from conf_ini_g.phase.specification.config.main import config_t as specification_t
from conf_ini_g.phase.untyped.parameter import ValueUnitAndComment
from conf_ini_g.phase.untyped.section import STD_UNIT_CONVERSIONS


def InitialUnits(specification: specification_t, /) -> tuple[dict[str, float], bool]:
    """
    Specification can override standard units.
    """
    units = dict(STD_UNIT_CONVERSIONS)
    if spec_has_units := (INI_UNIT_SECTION in specification):
        units.update(specification[INI_UNIT_SECTION].AsDict())

    return units, spec_has_units


def UpdateUnits(
    units: dict[str, float], name: str, value: str, /
) -> tuple[float | None, list[str]]:
    """"""
    issues = []

    if name in STD_UNIT_CONVERSIONS:
        return None, [f'Standard unit "{name}" cannot be overriden.']

    value, *_ = ValueUnitAndComment(value, INI_COMMENT_MARKER)
    try:
        value = float(value)
    except ValueError:
        return None, [f"/{INI_UNIT_SECTION}.{name}/ Invalid value: {value}."]

    units[name] = value

    return value, issues

==> conf-ini-g-2023.27.tar.gz: conf-ini-g-2023.27/conf_ini_g/phase/typed/section/units.py <==
# Copyright CNRS/Inria/UNS
# Contributor(s): Eric Debreuve (since 2021)
#
# eric.debreuve@cnrs.fr
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from conf_ini_g.interface.storage.parameter import INI_COMMENT_MARKER
from conf_ini_g.interface.storage.section import INI_UNIT_SECTION
from conf_ini_g.phase.specification.config.main import config_t as specification_t
from conf_ini_g.phase.untyped.parameter import ValueUnitAndComment
from conf_ini_g.phase.untyped.section import STD_UNIT_CONVERSIONS


def InitialUnits(specification: specification_t, /) -> dict[str, float]:
    """
    Specification can override standard units.
    """
    units = dict(STD_UNIT_CONVERSIONS)
    if INI_UNIT_SECTION in specification:
        units.update(specification[INI_UNIT_SECTION].AsDict())

    return units


def UpdateUnits(
    units: dict[str, float], name: str, value: str, /
) -> tuple[float | None, list[str]]:
    """"""
    issues = []

    if name in STD_UNIT_CONVERSIONS:
        return None, [f'Standard unit "{name}" cannot be overriden.']

    value, *_ = ValueUnitAndComment(value, INI_COMMENT_MARKER)
    try:
        value = float(value)
    except ValueError:
        return None, [f"/{INI_UNIT_SECTION}.{name}/ Invalid value: {value}."]

    units[name] = value

    return value, issues
